Artifact for
"Replicating RESTART with Prolonged Retrials: An Experimental Report"
by Carlos E. Budde and Arnd Hartmanns,
TACAS 2021.
=====================================================================


## Minimal instructions

Copy the "artifact" folder onto a Debian-based Linux system (that has build-essentials and cmake installed), and execute the script "run_artifact.sh":

$ cd artifact; ./run_artifact.sh

NOTE: If working with virtual machines and shared folders, please copy the archive containing the artifact into a folder inside of the machine and extract it there; otherwise (e.g. if working directly in a shared folder), symbolic links contained in the archive may not work correctly.

Instructions to perform a quicker test run are included towards the end of this file.

---------------------------------------------------------------------


## Contents of the artifact

- This Readme.txt file
- The License.txt file with the distribution license of the artifact
- The main execution script
  * in file "run_artifact.sh"
- Installation utilities for the code and software tools:
  * in folder "setup"
- All models and scripts for experimental reproduction
  * in folder "experiments"
- The logs of the runs used as results reported in the paper
  * in folder "paper_experiments_result_logs"
- The source code of different implementations of RESTART with prolonged retrials (cf. Algorithm 1 in the paper)
  * in folder "RESTART_algorithm_variants"


## Environment and requirements

The artifact is prepared to run in Debian-based Linux distributions; it has been tested in Ubuntu 20.04. The GNU essentials for C++ compilation must be available, i.e. the GNU compiler collection (gcc, g++) and Cmake version 2.8.13 or above.

You can download a VirtualBox (https://www.virtualbox.org/) VM image that satisfies all these requirements from:
https://zenodo.org/record/4041464#.X5GeOyVS-EJ
We refer to this virtual machine in its standard configuration with 1 processor and 2 GB RAM as the "TACAS VM".

Note that experimental runtimes depend strongly on the hardware and virtualisation employed. The times in seconds presented in Table 1 in the paper correspond to executions on two different (non-virtual) Intel CPUs:
- the adapted code, and FIG, ran on a Xeon E5-2630 v3 (2.4-3.2 GHz)
- the modes tool ran on a Core i7-4790 (3.6-4.0 GHz, 4 physical/8 logical cores)\
Executions in the TACAS VM typically involve runtimes considerably higher than those reported in Table 1. The number of samples and the estimated values, however, should remain comparable to Table 1.


## Files for inspection

The full output of the code/tool executions used to compose Table 1 in the paper is in folder "paper_experiments_result_logs".

A contribution of the paper is the iterative pseudocode of the RESTART algorithm with prolonged retrials of depth j, RESTART-Pj. In folder "RESTART_algorithm_variants" we provide different variants of source code that implement this pseudocode:

- "RESTART_Jose_original.cpp"
  * Original C++ source code provided to us by J. Villén-Altamirano, containing a recursive implementation of RESTART-P4 and an embedded 2-tandem Jackson queue model.
  * See the procedure "Restart(...)" in line 1055.
- "RESTART_Jose_adapted.cpp"
  * Our adaptation of the above, used to produce the results presented in the paper.
  * See the procedure "Restart(...)" in line 938.
- "RESTART_FIG.{h,cpp}"
  * C++ header and source files from the FIG tool that implement the iterative variant of RESTART-Pj presented as Algorithm 1 in the paper.
  * See the procedure `RESTART_run(...)` in line 354 of RESTART_FIG.cpp.
- "RESTART_modes_iterative.cs"
  * C# implementation of the iterative variant of RESTART-Pj, tested with the modes tool.
  * See the procedure "PerformRun()" in line 38.
- "RESTART_modes_recursive.cs"
  * C# implementation of the recursive variant of RESTART-Pj, used by default in the modes tool.
  * See the procedure "PerformRun()" in line 40.

Note that in the original and adapted code by J. Villen-Altamirano, RESTART-Pj is implemented recursively. This is also the implementation adopted by the modes tool in the Modest Toolset. FIG uses an iterative version of RESTART-Pj that is close to Algorithm 1 in the paper. The C# iterative version was also implemented in modes (before we obtained the original code), and was tested to produce the same results as the recursive variant. We chose to adopt the recursive variant in modes due to its more concise code and lower memory usage; the number of levels should in practice always be low enough to not run out of call stack space.


## Experiment reproduction

Running this artifact will reproduce the results presented in the paper. The entry point is the executable script "run_artifact.sh". To run the artifact, open a terminal, move into the "artifact" directory (where this "Readme.txt" file resides), and execute the script:
$ cd artifact; ./run_artifact.sh

This will perform the following steps:

1. Install the Modest Toolset (which contains the modes tool)
   * installation requires root privileges: please enter the password when prompted
   * if you are running this in the TACAS VM, the password is "tacas21"
   * if a working installation of modes is found, this step is omitted
2. Install the FIG tool in the environment
   * installation requires root privileges
   * external dependencies might be needed: agree to installing these if prompted
   * if a working installation of FIG is found, this step is omitted
3. Perform the experimental replication:
   * execute "experiments/run_Jose.sh" to reproduce the results for the adapted code
   * execute "experiments/run_modes.sh" to reproduce the results for the modes tool
   * execute "experiments/run_FIG.sh" to reproduce the results for the FIG tool

Step 3 runs the code/tools on the models presented in the paper. They all build confidence intervals using Student's-T quantiles that estimate the requested steady-state metrics. We run the code/tools to build confidence intervals with 95% confidence level and a relative error of 0.005 for the 2-tandem Jackson queue model, and relative error 0.1 for the other four models. These values follow the setup used in the original experiments by J. Villén-Altamirano referenced in the paper.

Note that both in the paper and in this artifact we report point estimates as results. We do not report confidence intervals because, with the information above, the confidence interval can be computed from the point estimate alone. Still, the confidence intervals can be read from the code/tool output logs included in folder "paper_experiments_result_logs", if the user so desires. Note also that the FIG tool takes as parameter the full (relative) confidence interval width, sometimes called 'precision'. This precision is thus twice the value of the relative error (which in turn is the half-width of the confidence interval).


### Results inspection

When each experiment from the reproduction package has finished (adapted code, modes tool, FIG tool), summary tables are created in the location indicated in the terminal, e.g.: 'Summary tables saved in /home/tacas21/artifact/experiments/./results/modes/Oct15_11:22:33/summary.txt'

These tables can be inspected and compared against the results shown in Table 1 on page 5 of the paper. An extract of a table looks like this:

> METHOD		ESTIMATE	#(SAMPLES)	RUNTIME in seconds
>
> 3tandem_queue_Jackson  |  CI:  .95 .2
>
> restart0		4.54e-15	 (118)		 40.43
> restart0		4.53e-15	 (89)		 51.22
>
> restart1		4.61e-15	 (70) 		 38.17
> restart1		5.26e-15	 (95)		 46.50
>
> restart2		4.14e-15	 (37) 		 32.70
> restart2		4.83e-15	 (40)		 44.21
>
> 2tandem_queue_Jackson  |  CI:  .95 .01
>
> restart0		4.07e-15	 (983)		435.97
> restart0		4.89e-15	 (777)		398.62

The results are grouped by model:

- The first column indicates the variant of RESTART-Pj used.
- The second column gives the point estimate computed (this characterises a confidence interval, see above)
- The third column gives the number of samples needed to converge to that estimate
- The fourth column gives the computation time in seconds that was needed
- "CI" indicates the confidence level and relative error used to determine convergence (the FIG tool reports precision = 2 x relative error)

Consecutive rows of the same RESTART-Pj variant indicate experiment repetitions. These are runs with the same configuration but different seeds for the RNG: the experiments of the first row use the RNGSEED=0, the second row use RNGSEED=1, and so on. Such choice of RNG seeds was made for experimental reproduction. However:

- The code by J. Villen-Altamirano uses the C standard library ('#include <cstdlib>') for random number generation:
  * this employs 'srand()' for RNG seeding, which produces the same sequence for the inputs '0' and '1' (to understand why, type 'man 3 srand' in a Linux terminal);
  * therefore, the results for the first two repetitions of this code produced exactly the same estimate and used the same number of samples;
  * this was not our intention - we only realised it when preparing the artifact; to make all repetitions of this code distinct, change '$REP' to '$((REP+1))' in the make line of the 'run_Jose.sh' script (i.e. the line that reads 'make $MV RNGSEED=$REP 2>&1 >/dev/null;').

- The FIG tool uses the '<random>' header from the C++ STL:
  * the RNG used is a 64-bit Mersenne Twister ('std::mt19937_64');
  * to run (stochastic) simulations, FIG uses this RNG to sample values from distributions such as exponential, Erlang, etc., also provided by the C++ STL;
  * although the Mersenne Twister RNG is deterministic for a given seed and its behaviour is fully specified by the C++ standard, the distributions are not (see e.g. https://stackoverflow.com/q/48730363);
  * so even for a fixed RNG seed, the rare-event values estimated by FIG may not match exactly the numbers obtained during our experiments for the paper (this was corroborated when running FIG in the TACAS VM).

- The modes tool runs simulations in multiple concurrent threads:
  * this adds a source of non-determinism, as the OS scheduler determines which thread is allocated to each CPU, and thus we do not specify seeds;
  * it is thus infeasible to reproduce exactly the values estimated by modes during experimentation for the paper;
  * still, just like with FIG, the estimates and number of samples produced by modes in this artifact should show the same trend as the results presented in Table 1 in the paper.

In summary, this artifact will NOT produce the numbers that we obtained in our experiments. It should produce numbers that are within the statistical margin of error w.r.t. our results; note the requested relative errors and confidence levels.


### Smaller and faster run

By default, in step 3 above, the artifact performs a full experimental reproduction. This can take some time. In our setting with high-end CPUs with several parallel cores and no virtualisation, some experiments for the 7-nodes non-Jackson network ran for approximately 8 h (each), and most experiments with the FIG tool for the 2-tandem Jackson queue (which require a relative error of 0.005) were aborted after exceeding a 15 h timeout.

It is also possible to execute a smaller version of our experiments: this will not replicate the results from the paper, but it can still convey a fair notion of the general results. Such faster convergence is achieved by:

- performing a single repetition only for each model,
- testing only the RESTART-P0 and -P2 prolongations,
- for FIG, relaxing the relative error to 0.2 for all models (caution: this means that the estimates may be far from the true values and those reported in the paper), and
- for modes, relaxing the relative error to 0.1 for the 2-tandem queue model.

To select this run type, set to '1' the value of the variable 'TEST_RUN' at the top of the main script "run_artifact.sh".
In our tests in the TACAS VM, the subsequent script execution took a little over 2 hours to complete on a host system with an Intel Core i5-6600T CPU.


### Troubleshooting artifact setup in the TACAS VM

If you are running this artifact in the TACAS VM, a daemon for unattended upgrades may lock the package manager on startup. This interferes with the installation of deb packages, required to satisfy the dependencies of the FIG tool.

Thus, if during installation you see the following error message:
'E: Could not get lock /var/lib/apt/lists ...'
then you will have to manually kill the "unattended-upgr" process prior to executing the artifact. Note that this requires root privileges, and you may need to do it twice in a row!
