//==============================================================================
//
//  Copyright
//
//  Jos Villn-Altamirano <jose.villen@upm.es> (Universidad Politcnica de Madrid)
//
//------------------------------------------------------------------------------
//
//  This file is part of the artifact created for the replication paper
//  "Replicating RESTART with Prolonged Retrials: An Experimental Report",
//  by Carlos E. Budde and Arnd Hartmanns (the "authors").
//  The source code was provided by Jos Villn-Altamirano (the "owner").
//
//==============================================================================

#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>

#include <string>
#include <iostream>
#include <fstream>
using namespace std;
#include <sstream>

/*************************************************************************/
/*********************** PARAMETROS GENERALES ****************************/
/*************************************************************************/
#define TRUE  1
#define FALSE 0
#define MAX_L 4096
/*************************************************************************/
/*********************** PARAMETROS DE SIMULACION ************************/
/*************************************************************************/
#define Num_Servidores 2 // = 0..4
const int Tasa_Llegada [Num_Servidores] = {1, 0};
const int Tasa_Servicio[Num_Servidores] = {2,3};
#define Num_Simulaciones 1 // NUMERO DE SIMULACIONES A COMPARAR
#define Num_Ejecuciones 1 // NUMERO DE EJECUCIONES A REALIZAR
//------------------------------------------------------------------------------------------------------------------------------------
 /* Matriz de transiciones entre nodos*/
const double prob_transito[Num_Servidores][Num_Servidores + 1] =
 { 
	{ 0.0 , 0.0, 1.0 },
	{1.0 , 1.0 , 1.0 }};
/*   S     1     2      */
/* probabilidades acumuladas de transito entre nodos-servidores*/
/* Se pone una columna mas para guardar la probabilidad de que no vaya */
/* a ningun servidor (probabilidad de salir de la red por ese servidor)*/
//------------------------------------------------------------------------------------------------------------------------------------
/* Caracteriaticas de la muestra */
#define MAXDOUBLE 100000000
#define Tam_Muestra 200000 /* Tamao de cada una de las muestras*/
#define Reloj_Final 30000
#define Tam_Transitorio 40 /* Llegadas a partir de las cuales termina el transitorio*/
//------------------------------------------------------------------------------------------------------------------------------------
/* Condiciones fin de simulacin*/
#define Max_Muestras 300 /* Limite por si no se logra un error relativo aceptable en un numero finito de muestras*/
#define Max_Error 0.005     /* Error relativo MINIMO para el cual termina la simulacin */
#define Calcula_Error 5   /* Numero de muestras a partir de las que empieza a calcular el error relativo */
/* Valor de t para el calculo del error */
const double Ta[56] = {2.36,2.36,2.36,2.36,2.36,2.36,2.36,2.36,2.31,2.26
					  ,2.23,2.20,2.18,2.16,2.14,2.13,2.12,2.11,2.10,2.09
					  ,2.09,2.08,2.07,2.07,2.06,2.06,2.06,2.05,2.05,2.04
					  ,2.04,2.04,2.04,2.04,2.03,2.03,2.03,2.03,2.02,2.02
					  ,2.02,2.02,2.02,2.02,2.02,2.02,2.01,2.01,2.01,2.01
					  ,2.01,2.01,2.01,2.01,2.01,2.01
					  }; 
//------------------------------------------------------------------------------------------------------------------------------------
/* Probabilidad a calcular */
#define Servidor_Simulado 2     /* Servidor del que se realiza la simulacin */
#define Clientes_En_Servidor 30 /* Num de clientes para calcular la probabilidad */
/* Se calcula la siguiente probabilidad               */
/*       P(Servidor_Simulado >= Clientes_En_Servidor) */
//------------------------------------------------------------------------------------------------------------------------------------

#define MaxUmbralesBajadosSplitting 14

/*************************************************************************/
/*********************** PARAMETROS DEL METODO RESTART *******************/
/*************************************************************************/
#define Numero_Umbrales 32 /* Umbrales  */ 
const int Umbrales[Numero_Umbrales + 1]={0 ,2 ,3 ,4 ,5, 6 ,7 ,8 ,9 ,10
										,11,12,13,14,15,16,17,18,19,20
										,21,22,23,24,25,26,27,28,29,30
          								,31,32 ,33//,36,37,38,39,40
          								//,41,42,43,44,45,46,47,48,49,50
										//,51,52,53,54,55,56,57,58,59,60
          								//,61,62,63,64,65,66,67,68,69,70
										   								};
//------------------------------------------------------------------------------------------------------------------------------------          								
 /*Se considera que el umbral de la primera zona es 0 y se incluye en el array*/
const int Repeticiones[Numero_Umbrales + 1]={1,2,2,2,2,2,2,3,3,3
											,3,3,3,3,3,2,3,3,3,2
											,3,3,3,3,3,3,3,3,3,3
                                            ,3,3,3
                                            };
const int Repeticiones1[Numero_Umbrales + 1]={1,2,3,2,3,3,2,3,3,3
											,2,3,3,3,3,3,3,3,3,2
											,3,3,3,3,3,3,3,3,3,3
                                            ,3,3,3
											};
const int Repeticiones2[Numero_Umbrales + 1]={1,2,3,2,3,3,2,3,3,3
											,2,3,3,3,3,3,3,3,3,2
											,3,3,3,3,3,3,3,3,3,3
                                            ,3,3,3
											};																						
//------------------------------------------------------------------------------------------------------------------------------------											 
// VALOR DE A para el CALCULO DE LA IMPORTANCIA
const double a[Num_Servidores]={ 0.63, 1};											 
 //------------------------------------------------------------------------------------------------------------------------------------

double Media_Pi[Numero_Umbrales + 1][Num_Simulaciones];
double ri[Numero_Umbrales][Num_Simulaciones];
int Ri[Numero_Umbrales][Num_Simulaciones];
double extremo_inf[Num_Simulaciones];
double extremo_sup[Num_Simulaciones];
double Error_Relativo[Num_Simulaciones]; /* Calculo del error relativo */
int Van_Num_Muestras[Num_Simulaciones];   /* Nmero de muestras que se han realizado */
double Media[Num_Simulaciones]; /* Media de la muestras (parmetro a estimar) */
int NumVecesSinClientes[Num_Simulaciones]; /* Numero de veces que se queda sin Clientes */
int NumVecesImportanciaCero; /* para su acumulacion parcial */

int VanPosicion;	  
/*****************************************************************************/
/*                       DECLARACION DE TIPOS                                */
/*****************************************************************************/
/***** Definicin del tipo elemento de una cola ***************/
typedef struct elemento_cola {
	double elemento;
	struct elemento_cola *sig;
}t_elem_cola;

/****** Definicion del tipo cola *********************************/
typedef t_elem_cola *t_cola;
//------------------------------------------------------------------------------------------------------------------------------------
/*** Definicin de variables de cada servidor *******/
typedef enum {Serv_Ocupado
			 ,Serv_Libre
} estado_serv;
//------------------------------------------------------------------------------------------------------------------------------------
typedef struct servidor {
  /******* Variables de estado del servidor  ****************/
  estado_serv  Estado;      	/* Estado en el que se encuentra          */
  int          Num_Clientes;   	/* Peticiones en el servidor              */
  double	   Tiempos[Clientes_En_Servidor + 1];   /* Contadores de tiempo */
  double       Cliente;      /* Peticion que esta siendo servida */
  t_cola       Cola;         /* Cola de peticiones del servidor           */
  /********   Variables de tiempos de sucesos  **************/
  double       Llegada;     /* Proxima llegada al servidor                */
  double       Salida;      /* Proxima salida del servidor                */
  double       Suceso_Ant;  /* Reloj del ultimo suceso del servidor       */
  /***********    Variables estadisticas       ***************/
  long         Clientes_Servidos;
  double       Tiempo_Cola;
}Tipo_Servidor;
//------------------------------------------------------------------------------------------------------------------------------------
/***********  Definicion de de contadores de tiempo de cada umbral *******/
 typedef struct tiempos_umbrales {
//+2 para que no de error raro???
 double Tiempos[Numero_Umbrales + 1];/* tiempo que pasa en cada umbral */
 double reloj;     					 /* tiempo global de la simulacion */
 double suceso_ant; 				 /* tiempo en el que se entro en el umbral actual */
 int umbral_calculo[Numero_Umbrales + 1];
 }Tipo_Tiempo_Umbral;

/*****************************************************************************/
/*                   DECLARACION DE VARIABLES globales                       */
/*****************************************************************************/
double reloj;
long Clientes_Servidos; /* Total de clientes servidos por el sistema */
long RandSeed = 1; 		/* Semilla del generador de numeros aleatorios*/
double Probabilidades[Max_Muestras]; /* Resultado simulado de cada muestra */
double Probabilidades_Rep[Max_Muestras][Numero_Umbrales + 1]; /* Resultado de la probabilidad Ti */
double r[400];
double ant = 0.0;

/*****************************************************************************/
/*                   DECLARACION DE funciones                                */
/*****************************************************************************/


/*****************************************************************************/
/*                     OPERACIONES DE LA COLA                                */
/*****************************************************************************/

/// <summary>
/// FUNCION QUE PERMITE SABER SI UNA COLA ESTA VACIA
/// </summary>
/// <param name="cola">COLA DE LA QUE QUEREMOS SABER SI ESTA VACIA.</param>
/// <returns>TRUE/FALSE.</returns>
int Vacia (t_cola cola)
{
  return (cola==NULL);
}

/// <summary>
/// FUNCION QUE PERMITE OBTENER EL ELEMENTO DE UN ELEMENTO DE LA COLA ????
/// </summary>
/// <param name="cola">ELEMENTO DE LA COLA DE LA QUE QUEREMOS OBTENER ELEMENTO.</param>
/// <returns>EL ELEMENTO EN CUESTION.</returns>
double Cima (t_cola cola)
{
  double elem;

  elem= (*cola).elemento;
  return elem;
}

/// <summary>
/// FUNCION QUE PERMITE MOSTRAR LOS ELEMENTOS DE UNA COLA
/// </summary>
/// <param name="pcola">COLA EN LA QUE QUEREMOS DESENCOLAR UN ELEMENTO.</param>
/// <param name="elem">ELEMENTO DESENCOLADO.</param>
/// <returns>void</returns>
void Mostrar (t_cola cola)
{
	if (cola==NULL) printf("Vacia   \n");
	else  while (cola != NULL)
			{
		printf("Elemento  %f",(*cola).elemento);
		cola=(*cola).sig;
			}
printf("\n");
}


/// <summary>
/// FUNCION QUE PERMITE ENCOLAR UN NUEVO ELEMENTO EN LA COLA
/// </summary>
/// <param name="pcola">COLA EN LA QUE QUEREMOS ENCOLAR UN ELEMENTO.</param>
/// <param name="elem">ELEMENTO A ENCOLAR.</param>
/// <returns>void</returns>
void Encolar (t_cola *pcola,double elem)
{

  t_cola aux;
  if (*pcola==NULL)
  {
	 *pcola=(t_elem_cola *) malloc(sizeof(t_elem_cola));
	 aux=*pcola;	 
	 (*aux).elemento=elem;
	 (*aux).sig=NULL;
  	 //Mostrar(*pcola);	 
  }
  else
  {
	 aux=*pcola;
	 Encolar(&((*aux).sig),elem);
  	 //Mostrar(*pcola);	 	 
  }
}

/// <summary>
/// FUNCION QUE PERMITE DESENCOLAR UN ELEMENTO EN LA COLA
/// </summary>
/// <param name="pcola">COLA EN LA QUE QUEREMOS DESENCOLAR UN ELEMENTO.</param>
/// <param name="elem">ELEMENTO donde debemos dejar el DESENCOLADO.</param>
/// <returns>void</returns>
void Desencolar (t_cola *pcola,double *elem)
{
	t_cola aux;

	aux=*pcola;  		
	//Mostrar(*pcola);
  	*pcola=(*aux).sig;	
	//Mostrar(*pcola);  	
	*elem=(*aux).elemento;   	
  
//  *elem=aux->elemento; 
//  *pcola=aux->sig;
  
  free(aux);
}




/*****************************************************************************/
/*                        FUNCIONES GENERICAS                                    */
/*****************************************************************************/
/*void Muestra_Servidores (void)
{
int i;
  printf("RELOJ %2.2f  ",reloj);

  printf("\n");
  for (i=0;i<Num_Servidores;i++)
		 {
			 printf("Servidor %i Clientes %i ",i+1,Servidores[i].Num_Clientes);
			 printf("Salida %f  Llegada %f \n",Servidores[i].Salida,Servidores[i].Llegada);
		 }
  printf("\n");

		 getch();


}
  */
/*****************************************************************************/

/// <summary>
/// FUNCION QUE PERMITE CALCULAR LA funcion de IMPORTANCIA
/// </summary>
/// <param name="s1..7">VALOR DEL NUMERO DE CLIENTES EN CADA SERVIDOR.</param>
/// <returns>valor de la IMPORTANCIA</returns>
double Importancia(int s1,int s2,int s3,int s4,int s5, int s6, int s7)
{
	return (a[0] * s1) + (a[1] * s2) + (a[2] * s3) + (a[3] * s4) + (a[4] * s5) +
 			(a[5] * s6) + (a[6] * s7);
}

double Calcula_Importancia (Tipo_Servidor Servidores[])
{
	double Calculado = 0;
  	for (int i = 0; i < Num_Servidores; i++)
	{
		Calculado = Calculado + (a[i] * Servidores[i].Num_Clientes);	
	}
	
	if (Calculado == 0)
	{
		NumVecesImportanciaCero++;
	}
	
	return Calculado;	
}


double Generate(void)
{
 #define MODULUS         2147483647L
 #define MULTIPLIER      16807
 #define QUOTIENT        127773L
 #define REMAINDER       2836

 long Low = RandSeed % QUOTIENT;
 long Hi = RandSeed / QUOTIENT;
 long Test = MULTIPLIER * Low - REMAINDER * Hi;

 /*Update seed for next time*/
 if (Test > 0)
 {
	RandSeed=Test;
 }
 else
 {
	RandSeed = Test + MODULUS;
 }

 return ((double)RandSeed/MODULUS)  ;
 #undef MODULUS
 #undef MULTIPLIER
 #undef QUOTIENT
 #undef REMAINDER
}


double Exponencial(double lambda)
{
	double u;
	double res=-1.0;

	do
	{
 		u=rand()/(double)RAND_MAX;
 		if (u!=0)  
		 	res=1;
	} while (res<0) ;

	/*u=Generate();  */
	res=((-1/lambda)*log(u));
	return res;
}

double Erlang(int a,int b)
{
	double acum;
	int i;

	acum=rand()/(double)RAND_MAX;
	for (i=1; i<= (a-1);i++)
		acum=acum*(rand()/(double)RAND_MAX);
	return (-1/b*log(acum));
	// BUG: ^^^ should be "-1.0/b" and not "-1/b" to avoid integer division
}

double Student (int n)
{
	if (n <= 55) return Ta[n];
	else if ((n>=56) && (n<=80)) return 2.0;
		 else if ((n>=81) && (n<=105)) return 1.99;
				else if ((n>=106) && (n<=130)) return 1.98;
					  else if ((n>=131) && (n<=200)) return 1.97;
							  else return 1.96;
}

/*****************************************************************************/

void Inicializa_Servidores(Tipo_Servidor Servidores[])
{
	int i,j;
  	for (i = 0; i < Num_Servidores; i++)
	{
		Servidores[i].Estado = Serv_Libre;
		Servidores[i].Num_Clientes = 0;
		for (j = 0; j < Clientes_En_Servidor + 1 ; j++)
		  Servidores[i].Tiempos[j] = 0;
		Servidores[i].Cola=NULL;
		Servidores[i].Cliente = -1.0;
	  	//Servidores[i].Llegada = Erlang(1,Tasa_Llegada[i]);	
		Servidores[i].Llegada = Exponencial((double)Tasa_Llegada[i]); // CALCULA LA PRIMERA LLEGADA
		Servidores[i].Salida = 999999.99;/*numero alto para la primera comparacion*/
		Servidores[i].Suceso_Ant = 0.0;
		Servidores[i].Clientes_Servidos = 0;
		Servidores[i].Tiempo_Cola = 0.0;
	}	
}

void Inicializa_Despues_de_Transitorio(Tipo_Servidor Servidores[],Tipo_Tiempo_Umbral* Tiempos_umbral)
{
	int i,j;
  	for (i=0; i < Num_Servidores ;i++)
		 {
		  for (j=0; j < Clientes_En_Servidor + 1 ;j++)
			  Servidores[i].Tiempos[j] = 0.0;
		 }
		 
	for (i=0; i <= Numero_Umbrales ;i++)
		{
			(*Tiempos_umbral).Tiempos[i] = 0.0;
			(*Tiempos_umbral).umbral_calculo[i] = 0;				 
		}
	(*Tiempos_umbral).reloj = 0.0;
	(*Tiempos_umbral).suceso_ant = 0.0;

}


/*****************************************************************************/
/* MIRA en que servidor se realiza la proxima llegada y devuelve el tiempo en el que ocurre*/
// Y EL SERVIDOR EN EL QUE OCURRE (serv) !!!!
double Proxima_Llegada(int *serv,Tipo_Servidor Servidores[])
{
  double menor=MAXDOUBLE;
  int i;

  for (i = 0; i < Num_Servidores; i++)
	if (Servidores[i].Llegada < menor)
	{
		*serv=i+1;
		menor=Servidores[i].Llegada;
	}

  /** traza SI HAY ERROR Y ES MENOR QUE EL RELOJ ACTUAL !!!! */
  if (menor<reloj)
  {
  	printf("menor\n") ;
  }
 return menor;
}

/*MIRA en que servidor se realiza la proxima salida y devuelve el tiempo en el que ocurre*/
// Y EL SERVIDOR EN EL QUE OCURRE (serv) !!!!
double Proxima_Salida(int *serv,Tipo_Servidor Servidores[])
{
  double menor=MAXDOUBLE;
  int i;

  for (i = 0; i < Num_Servidores; i++)
	if ((Servidores[i].Salida < menor) && (Servidores[i].Num_Clientes > 0))
	 {
		*serv = i+1;
		menor = Servidores[i].Salida;
	 }
	 
	if (serv < 0)
	{
		printf("serv menor que 0\n") ;	
	}
	 /****traza*/
	if (menor<reloj)
	{
		printf("menor\n") ;
	}
	return menor;
}


/*****************************************************************************/
/* Dado un servidor de salida gestiona, en base a matriz de probabilidades prob_transito,hacia donde va */
/* Salida: Numero de servidor o "0" si va fuera del sistema */
int SeleccionaDestino(int serv)
{
  int i = 0,destino = -1;
  double prob;

  prob=rand()/(double)RAND_MAX;

  while ((destino == - 1)&&(i <= Num_Servidores))
	{
	  if (prob_transito[serv-1][i]>=prob)
		{
			destino = i;
		}

	  i++;
	}
/*destino 0 es que va fuera del sistema*/

  return destino;

}


/*****************************************************************************/
double Calcula_Indice_Umbral(int factor)
{
  int i;
  double solucion = 1;

  for (i = 0; i <= factor; i++)
  {
  	if (VanPosicion == 0)
	 	solucion = solucion * Repeticiones[i];  	
  	if (VanPosicion == 1)
	 	solucion = solucion * Repeticiones1[i];  	
  	if (VanPosicion == 2)
	 	solucion = solucion * Repeticiones1[i];  	
  }	 
  return solucion;

}
/*****************************************************************************/
double Calcula_Indice_Umbral_Despues(int umbral_calculo, int factor)
{
  int i;
  double solucion = 1;

  for (i = 0; i <= umbral_calculo; i++)  
  {
  	if (VanPosicion == 0)
	 	solucion = solucion * Repeticiones[i];  	
  	if (VanPosicion == 1)
	 	solucion = solucion * Repeticiones1[i];  	
  	if (VanPosicion == 2)
	 	solucion = solucion * Repeticiones1[i];  	
  }	 
  return solucion;

}

/*****************************************************************************/
void Gestiona_Llegada(int serv
					, int De_Fuera
					, int umbral_act
					, int umbral_inicial
					, Tipo_Servidor Servidores[]
					, Tipo_Tiempo_Umbral* Tiempos_umbral
					, int EsSplittingYBajoDelUmbral
					)
{
	int Indice_Tiempos; /* En caso de que haya mas de Clientes_En_Servidor en el nodo         */
	                 /* el tiempo Se le suma al mayor, ya que la probabilidad va ser ">="  */
	
	double indice_umbral; /* Limita en caso de que el num de clientes sea mayor que el que se quiere calcular en la probabilidad */
	
	if (Servidores[serv-1].Num_Clientes > Clientes_En_Servidor)
	Indice_Tiempos = Clientes_En_Servidor;
	else
	Indice_Tiempos = (int)Servidores[serv-1].Num_Clientes;


	/*Actualiza reloj si la llegada es de fuera(si no ya se ha actualizado en salidas)*/
	if (De_Fuera)
	{
	  /*actualiza el tiempo global de simulacion*/
	  /*es el tiempo del suceso actual menos el del ultimo que esta en reloj*/
	  /*nuevo*/
	  (*Tiempos_umbral).reloj = (*Tiempos_umbral).reloj + (Servidores[serv-1].Llegada - reloj);
	
	  /*actualiza el reloj*/
	  reloj = Servidores[serv-1].Llegada;	
	}

	/******traza**/
	/*		printf("Llegada a %i en %f \n",serv,reloj);*/

	/* Calcular Ti */
	/*nuevo*/
	if (EsSplittingYBajoDelUmbral == FALSE)
	{
		indice_umbral = Calcula_Indice_Umbral(umbral_act);
		if (indice_umbral == 0) /*****traza*/
		 {
			indice_umbral = Calcula_Indice_Umbral(umbral_act);
		 }
	}	 
	
	if (EsSplittingYBajoDelUmbral == TRUE)
	{
		//indice_umbral = Calcula_Indice_Umbral_Despues(umbral_inicial,0);
		indice_umbral = Calcula_Indice_Umbral(umbral_inicial);		
		//if (indice_umbral == 0) /*****traza*/
		// {
		//	indice_umbral = Calcula_Indice_Umbral_Despues(umbral_act,0);
		// }
	}	 
	
	Servidores[serv - 1].Tiempos[Indice_Tiempos] = Servidores[serv-1].Tiempos[Indice_Tiempos] +
		  ((reloj - Servidores[serv - 1].Suceso_Ant)/indice_umbral);

	/* Comprobar estado */
	/*Si ocupado encolar */
	/*Si libre pasa a cliente,calcula salida,Actualiza estado*/
	if (Servidores[serv-1].Estado == Serv_Ocupado)
 	{
		/*Encolar(&Servidores[serv-1].Cola,reloj);*/
	 	reloj = reloj;  /*se pone por poner algo en vez de encolar*****/
 	}
  	else
 	{
		Servidores[serv-1].Estado = Serv_Ocupado;
		Servidores[serv-1].Cliente = reloj;
		Servidores[serv-1].Salida = reloj + Exponencial(Tasa_Servicio[serv-1]);
	}
  
 	/* Incrementa Num_Clientes*/
	Servidores[serv-1].Num_Clientes++;

	/*Actualizarsuceso anterior*/
	Servidores[serv-1].Suceso_Ant = reloj;

	/* Calcular proxima Llegada si la llegada fue de fuera*/
	if (De_Fuera)
	{
	  /*Servidores[serv-1].Llegada=reloj + Erlang(1,Tasa_Llegada[serv-1]);******/
	  Servidores[serv-1].Llegada = reloj + Exponencial(Tasa_Llegada[serv-1]);
	}
}


/*****************************************************************************/
/* Gestiona la salida de un mensaje de un serv y calcula hacia donde va el siguiente mensaje */
void Gestiona_Salida(int serv
					, int umbral_act
					, int umbral_inicial
					, Tipo_Servidor Servidores[]
					, Tipo_Tiempo_Umbral* Tiempos_umbral
					, int EsSplittingYBajoDelUmbral					
					)
					
					
{
	int destino;
	double saliente;
	int Indice_Tiempos; /* En caso de que haya mas de Clientes_En_Servidor en el nodo         */
						/* el tiempo se le suma al mayor, ya que la probabilidad va ser ">="  */

	double indice_umbral;/* Limita en caso de que el num de clietes sea mayor que el que se quiere calcular en la probabilidad */
						 /* numero que divide el tiempo en funcion del umbral en el que este*/

	if (Servidores[serv-1].Num_Clientes > Clientes_En_Servidor)
		Indice_Tiempos = Clientes_En_Servidor;
	else
		Indice_Tiempos = (int)Servidores[serv-1].Num_Clientes;

 	saliente = Servidores[serv-1].Cliente;
 	destino = SeleccionaDestino(serv);

	/*Actualiza el reloj global y el parcial de simulacion*/
	/*nuevo*/
	(*Tiempos_umbral).reloj = (*Tiempos_umbral).reloj + (Servidores[serv-1].Salida - reloj);
	reloj = Servidores[serv-1].Salida;

	/*******traza******/
	/*	 printf("Salida de %i a %i en %f  \n",serv,destino,reloj);*/

	/*Si hay alguien en cola lo saca  y lo mete en cliente, si no pone estado a libre */
	if (Servidores[serv-1].Num_Clientes > 1)
 	{
		/* Desencolar(&Servidores[serv-1].Cola,&Servidores[serv-1].Cliente);*/
		saliente=saliente;
 	}
 	else
 		{
 		Servidores[serv-1].Estado=Serv_Libre;
 		}

 	/*Actualiza el tiempo que el cliente est en cola (si sacas a alguien de la cola)*/
 	/*Actualiza contadores de tiempo Ti*/
 	/*nuevo*/
	if (EsSplittingYBajoDelUmbral == FALSE)
	{ 	
	 	indice_umbral=Calcula_Indice_Umbral(umbral_act);
	 	if (indice_umbral == 0)/*****traza*/
	 	{
			indice_umbral=Calcula_Indice_Umbral(umbral_act);
	 	}
	}	
	
	if (EsSplittingYBajoDelUmbral == TRUE)
	{ 	
		//indice_umbral = Calcula_Indice_Umbral_Despues(umbral_inicial,0);	
	 	indice_umbral = Calcula_Indice_Umbral(umbral_inicial);
	 	//if (indice_umbral == 0)/*****traza*/
	 	//{
		//	indice_umbral=Calcula_Indice_Umbral(umbral_act);
	 	//}
	}		
	
	Servidores[serv - 1].Tiempos[Indice_Tiempos] = Servidores[serv - 1].Tiempos[Indice_Tiempos] 
	 								+ ((reloj - Servidores[serv - 1].Suceso_Ant)/indice_umbral);

	/*Actualiza numclientes */
	Servidores[serv-1].Num_Clientes--;

	/*Actualiza Proxima salida con tiempo de servicio del servidor(si hay cliente) */
	if (Servidores[serv-1].Num_Clientes > 0)
	{
		Servidores[serv-1].Salida = reloj + Exponencial(Tasa_Servicio[serv-1]);
	}

	/*Actualiza Suceso Anterior*/
	Servidores[serv-1].Suceso_Ant = reloj;

	/*Actualiza clientes servidos*/
	Servidores[serv-1].Clientes_Servidos++;

	/*Si procede, gestionar entrada al servidor destino*/
	/*si no incrementar Clientes_Servidos si hay salida fuera del sistema*/
	if (destino!=0)
	{
		int Viene_De_Fuera = FALSE;
		//Gestiona_Llegada(destino,Viene_De_Fuera,umbral_act,Servidores,Tiempos_umbral);
		Gestiona_Llegada(destino
						,Viene_De_Fuera
						,umbral_act
						,umbral_inicial
						,Servidores
						,Tiempos_umbral
						,EsSplittingYBajoDelUmbral);		
	}
	/*else Clientes_Servidos++;****/
}

/*****************************************************************************/
double Tiempo_Restart(Tipo_Tiempo_Umbral* Tiempos_umbral)
{
 double solucion=0,indice_umbral;
 int i;
 //double indice_umbral;

	for (i=0;i <= Numero_Umbrales;i++)
	{
		indice_umbral=Calcula_Indice_Umbral(i);
		solucion=solucion + ((*Tiempos_umbral).Tiempos[i]/indice_umbral);
		/*if (i==0)
		{
			solucion=solucion + ((*Tiempos_umbral).Tiempos[i]);
		}
		else
		{
			solucion=solucion + ((*Tiempos_umbral).Tiempos[i]/ (double)Repeticiones[i]);
		}*/
	 }

	 return solucion;
}
/*****************************************************************************/
double Tiempo_Splitting(Tipo_Tiempo_Umbral* Tiempos_umbral
			)
{
	double solucion = 0;
	double indice_umbral;
	double indice_umbral_despues;	
	int i;

	for (i = 0;i <= Numero_Umbrales; i++)
	{
		indice_umbral = Calcula_Indice_Umbral(i);
		solucion = solucion + ((*Tiempos_umbral).Tiempos[i] / indice_umbral);
		/*if (i==0)
		{
			solucion=solucion + ((*Tiempos_umbral).Tiempos[i]);
		}
		else
		{
			solucion=solucion + ((*Tiempos_umbral).Tiempos[i]/ (double)Repeticiones[i]);
		}*/
	 }

	 return solucion;
}


/*****************************************************************************/
double Calcula_Probabilidad(Tipo_Servidor Servidores[]
							,Tipo_Tiempo_Umbral* Tiempos_umbral)
{
	double tiempo;
	
	tiempo = Tiempo_Restart(Tiempos_umbral);
	tiempo = Reloj_Final;
	
	return Servidores[Servidor_Simulado-1].Tiempos[Clientes_En_Servidor] / tiempo;
}
/*****************************************************************************/
double Calcula_Probabilidad_Splitting(Tipo_Servidor Servidores[]
									,Tipo_Tiempo_Umbral* Tiempos_umbral
									)
{
	double tiempo;
	tiempo = Tiempo_Splitting(Tiempos_umbral);	

	tiempo = Reloj_Final;
	
	return Servidores[Servidor_Simulado-1].Tiempos[Clientes_En_Servidor] / tiempo;
}
/*****************************************************************************/
double Calcula_Probabilidad_Ti(Tipo_Tiempo_Umbral* Tiempos_umbral
							,int umbral)
{
	double tiempo;
	double indice_umbral;
	double suma_tiempos = 0;
	int i;

 	tiempo = Tiempo_Restart(Tiempos_umbral);
	//i=umbral porque los umbrales van de 0 a n (el 0 est incluido)
	for (i = umbral; i <= Numero_Umbrales; i++)
	{
    	indice_umbral = Calcula_Indice_Umbral(i); 
    	suma_tiempos += (*Tiempos_umbral).Tiempos[i]/indice_umbral; 
	}
 	return suma_tiempos / tiempo;
}

/*****************************************************************************/
double Calcula_Probabilidad_Ti_Splitting(Tipo_Tiempo_Umbral* Tiempos_umbral
										,int umbral)
{
	double tiempo;
	double indice_umbral;
	double indice_umbral_despues;	
	double suma_tiempos=0;
	int i;

	tiempo = Tiempo_Splitting(Tiempos_umbral); 		
	
	//i=umbral porque los umbrales van de 0 a n (el 0 est incluido)
	for (i = umbral; i <= Numero_Umbrales; i++)
	{
    	indice_umbral = Calcula_Indice_Umbral(i); 
    	suma_tiempos += (*Tiempos_umbral).Tiempos[i] / indice_umbral; 
	}
	
 	return suma_tiempos / tiempo;
}

/*****************************************************************************/


double Calcula_Media(int n)
{
  int i;
  double suma=0;

  for (i=0;i<n;i++)
	 suma += Probabilidades[i];

  return suma/(double)n;
}

/*****************************************************************************/


double Calcula_Media_Umbral(int n,int umbral)
{
  int i;
  double suma=0;

  for (i=0;i<n;i++)
	 suma += Probabilidades_Rep[i][umbral-1];

  return suma/(double)n;
}

/*****************************************************************************/

double Calcula_s2(int n,double med)
{
	int i;
	double suma=0;

  	for (i=0;i<n;i++)
		suma += (Probabilidades[i]-med)*(Probabilidades[i]-med);

  	return suma/(n-1);
}

/*****************************************************************************/

double Calcula_Error_Relativo(double desv,double med,int n)
{
	return (Student(n) * (desv / sqrt(n)))/med;
}
/*****************************************************************************/
double Calcula_Intervalo_Confianza(double desv,int n)
{
	return (Student(n) * (desv / sqrt(n)) );
}
/*****************************************************************************/
void Imprime_Pantalla(int n, double Error, double med)
{
	printf("En %i muestras, error de %.4f con media %e  \n", n, Error, med);
}
/*****************************************************************************/
void Transitorio(Tipo_Servidor Servidores[],Tipo_Tiempo_Umbral* Tiempos_umbral)
{
	int servidor; /* Servidor en el que ocurre el siguiente evento */
	int serv_salida = -1;
	int serv_entrada = -1;

	reloj = 0.0;
	Clientes_Servidos=0;

	Inicializa_Servidores(Servidores);
	/*Muestra_Servidores();*/
	int i;
	for (i=0; i <= Numero_Umbrales ;i++)
		{
			(*Tiempos_umbral).Tiempos[i] = 0.0;
			(*Tiempos_umbral).umbral_calculo[i] = 0;	
		}
	(*Tiempos_umbral).reloj = 0.0;
	(*Tiempos_umbral).suceso_ant = 0.0;		
	
	int Viene_De_Fuera = FALSE;

  	do
 	{	
 	    // SE CALCULA LA PROXIMA LLEGADA Y SALIDA Y SE MIRA A VER CUAL SE VA A DAR ANTES
		if (Proxima_Salida(&serv_salida,Servidores) <= Proxima_Llegada(&serv_entrada,Servidores))
		{
		  // SI LA Proxima_Salida es menor o igual que la Proxima_Llegada HACE SALIDA
		  servidor=serv_salida;
		  Gestiona_Salida(servidor
						  ,0
						  ,0
						  ,Servidores
						  ,Tiempos_umbral
						  ,FALSE);
		}
		else
		{
		  // SI LA Proxima_Salida es MAYOR que la Proxima_Llegada HACE LLEGADA	
		  servidor=serv_entrada;
		  Viene_De_Fuera = TRUE;
		  Gestiona_Llegada(servidor
		  					,Viene_De_Fuera
							,0
							,0
							,Servidores
							,Tiempos_umbral
							,FALSE);
		  Clientes_Servidos++;
		}

	} while (Clientes_Servidos < Tam_Transitorio);

 	Inicializa_Despues_de_Transitorio(Servidores,Tiempos_umbral);
}
/*****************************************************************************/
void Lista_Sucesos_Nueva(Tipo_Servidor Servidores[])
{
	int i;

	// CALCULO NUEVA LLEGADA PARA CADA SERVIDOR
	// Y TAMBIEN NUEVA SALIDA SI YA HABIA CLIENTES EN EL SERVIDOR
	for (i = 0; i < Num_Servidores; i++)
 	{
	  	//Servidores[i].Llegada=reloj + Erlang(1,Tasa_Llegada[i]);
		Servidores[i].Llegada = reloj + Exponencial(Tasa_Llegada[i]);
		
	  	if (Servidores[i].Num_Clientes > 0)
			Servidores[i].Salida = reloj + Exponencial(Tasa_Servicio[i]);
 	}
}
/*****************************************************************************/
void Lista_Sucesos_Nueva_Splitting(Tipo_Servidor Servidores[])
{
	int i;

	// CALCULO NUEVA LLEGADA PARA CADA SERVIDOR
	// Y TAMBIEN NUEVA SALIDA SI YA HABIA CLIENTES EN EL SERVIDOR
	for (i = 0; i < Num_Servidores; i++)
 	{
	  	//Servidores[i].Llegada=reloj + Erlang(1,Tasa_Llegada[i]);
		Servidores[i].Llegada = reloj + Exponencial(Tasa_Llegada[i]);
		
	  	if (Servidores[i].Num_Clientes > 0)
	  		{
			Servidores[i].Salida = reloj + Exponencial(Tasa_Servicio[i]);
	  		}			
		else
			{
			Servidores[i].Salida = 999999.99; /*numero alto para la primera comparacion*/
			}
 	}
}
/*****************************************************************************/
void AcumulaTiemposUmbral(int Umbral, Tipo_Tiempo_Umbral* Tiempos_umbral)
{
	(*Tiempos_umbral).Tiempos[Umbral] = (*Tiempos_umbral).Tiempos[Umbral]
									      + ((*Tiempos_umbral).reloj - (*Tiempos_umbral).suceso_ant);
	(*Tiempos_umbral).suceso_ant = (*Tiempos_umbral).reloj;	
}
/*****************************************************************************/
void AcumulaTiemposUmbral_AlBajar(int Umbral
			, Tipo_Tiempo_Umbral* Tiempos_umbral_antes
			, Tipo_Tiempo_Umbral* Tiempos_umbral
			)
{
	(*Tiempos_umbral).Tiempos[Umbral] = (*Tiempos_umbral).Tiempos[Umbral]
									      + ((*Tiempos_umbral_antes).reloj - (*Tiempos_umbral_antes).suceso_ant);
	(*Tiempos_umbral).suceso_ant = (*Tiempos_umbral_antes).reloj;	
	}
/*****************************************************************************/
void AcumulaTiemposUmbral_DespuesDeBajar(int Umbral
			, Tipo_Tiempo_Umbral* Tiempos_umbral_antes
			, Tipo_Tiempo_Umbral* Tiempos_umbral
			)
{
	(*Tiempos_umbral).Tiempos[Umbral] = (*Tiempos_umbral).Tiempos[Umbral]
									      + ((*Tiempos_umbral).reloj - (*Tiempos_umbral).suceso_ant);
	(*Tiempos_umbral).suceso_ant = (*Tiempos_umbral).reloj;	
	
}
/*****************************************************************************/
void AcumulaTiemposUmbral_DespuesDeBajar_Final(int Umbral
			, Tipo_Tiempo_Umbral* Tiempos_umbral_antes
			, Tipo_Tiempo_Umbral* Tiempos_umbral
			)
{
	(*Tiempos_umbral).Tiempos[Umbral] = (*Tiempos_umbral).Tiempos[Umbral]
									      + ((*Tiempos_umbral).reloj - (*Tiempos_umbral).suceso_ant);
	(*Tiempos_umbral).suceso_ant = (*Tiempos_umbral).reloj;	
	
}
/*****************************************************************************/
void Restart(int umbral_act
			  , Tipo_Servidor Servidores[]
			  , Tipo_Tiempo_Umbral* Tiempos_umbral)
{
	int servidor; /* Servidor en el que ocurre el siguiente evento */
	int serv_salida = -1;
	int serv_entrada = -1;
	int i;
	// PARA GUARDAR el estado con el que entra al umbral Y LUEGO PODER RECUPERARLO
	Tipo_Servidor Servidor_Guardado[Num_Servidores];
	// PARA GUARDAR EL RELOJ CON EL QUE SE ENTRA AL UMBRAL Y LUEGO PODER RECUPERARLO
	double reloj_Guardado = 0;

	double importancia; /* guarda el calculo de la funcion de importancia */
	int repeticion = 1;   /* numero de repeticiones que se realizan en el umbral actual*/

 	/* guardo el estado actual */
	for (i = 0; i < Num_Servidores; i++)
	{		
		 Servidor_Guardado[i].Estado = Servidores[i].Estado;
		 Servidor_Guardado[i].Num_Clientes = Servidores[i].Num_Clientes;
		 Servidor_Guardado[i].Cliente = Servidores[i].Cliente;
		 Servidor_Guardado[i].Suceso_Ant = Servidores[i].Suceso_Ant;		 
	}
	reloj_Guardado = reloj;
	int Viene_De_Fuera = FALSE;

	// BUCLE PRINCIPAL HASTA QUE:
	// Clientes_Servidos alcance el Tamao de la Muestra 
	// o 
	// se alcance el numero de repeticiones por umbral
	do 
 	{
 	    // SE CALCULA LA PROXIMA LLEGADA Y SALIDA Y SE MIRA A VER CUAL SE VA A DAR ANTES 		
		if (Proxima_Salida(&serv_salida,Servidores) <= Proxima_Llegada(&serv_entrada,Servidores))
		{
		  // SI LA Proxima_Salida es menor o igual que la Proxima_Llegada HACE SALIDA			
		  servidor=serv_salida;
		  Gestiona_Salida(servidor
		  				  ,umbral_act
		  				  ,umbral_act
						  ,Servidores
						  ,Tiempos_umbral
						  ,FALSE);
		}
		else
		{
		  // SI LA Proxima_Salida es MAYOR que la Proxima_Llegada HACE LLEGADA				
		  servidor=serv_entrada;
		  Viene_De_Fuera = TRUE;
		  Gestiona_Llegada(servidor
		  					,Viene_De_Fuera
						    ,umbral_act
						    ,umbral_act
							,Servidores
							,Tiempos_umbral
							,FALSE);
		  Clientes_Servidos++;
		}

		/* calculo la funcion de importancia */
		// EN FUNCION DEL NUMERO DE CLIENTES EN CADA SERVIDOR
		importancia = Calcula_Importancia(Servidores);

		/* en funcion de la importancia cambio o no de umbral*/
		/* Cambio (umbral_act<Numero_Umbrales) por (umbral_act<=Numero_Umbrales)  22/01/2006    */
		if ((umbral_act < Numero_Umbrales) 
			 && (importancia > Umbrales[umbral_act + 1])
		   )		
		{
			// SI NO HE LLEGADO AL NUMERO TOTAL DE UMBRALES
			// Y
			// LA IMPORTANCIA CALCULADA ES MAYOR QUE LA DEL UMBRAL SIGUIENTE
			/* Sube de umbral*/
			AcumulaTiemposUmbral(umbral_act,Tiempos_umbral);
			/*Llamada recursiva para el siguiente nivel*/			
			Restart(umbral_act + 1, Servidores, Tiempos_umbral);
		}
		else 
		{
		//	if (importancia < Umbrales[umbral_act])
		if ( ( (umbral_act == 1) && (importancia < Umbrales[umbral_act])  )
            ||
                 ( (umbral_act == 2) && (importancia < Umbrales[umbral_act-1]) )  
            ||    
                 ( (umbral_act == 3) && (importancia < Umbrales[umbral_act-2]) )
            ||    
                ( (umbral_act == 4) && (importancia < Umbrales[umbral_act-3]) )
            ||                                                                                
			     ( (umbral_act > 4) &&  (importancia < Umbrales[umbral_act-4])  )
			    )  
				{
					// SI LA IMPORTANCIA CALCULADA ES MENOR QUE LA DEL UMBRAL ACTUAL
					// VER SI SE HA LLEGADO AL NUMERO DE REPETICIONES DEL UMBRAL
					/* Baja DEL umbral ACTUAL*/
					AcumulaTiemposUmbral(umbral_act,Tiempos_umbral);				

				 	if (repeticion < Repeticiones[umbral_act])
				 	{
				 		// SI NO HE LLEGADO AL NUMERO DE REPETICIONES DEL UMBRAL
						// recuperamos reloj y estado servidores AL ENTRAR AL UMBRAL
						// Y HACEMOS RE-SCHEDULING ANTES DE REPETIR ????
						reloj = reloj_Guardado;
						for (i = 0; i < Num_Servidores; i++)
						 {
							 Servidores[i].Estado = Servidor_Guardado[i].Estado;
							 Servidores[i].Num_Clientes = Servidor_Guardado[i].Num_Clientes;
							 Servidores[i].Cliente = Servidor_Guardado[i].Cliente;
						 	 Servidores[i].Suceso_Ant = Servidor_Guardado[i].Suceso_Ant;						
					 	}
					 	Lista_Sucesos_Nueva(Servidores);
				 	}
					repeticion++;
			   }
		}
			// SI NO ES NINGUNA DE LAS CONDICIONES ANTERIORES ES QUE HA LLEGADO AL NUMERO DE UMBRALES ????
	//} while ((Clientes_Servidos < Tam_Muestra) && (repeticion <= Repeticiones[umbral_act] )) ;
	} while (reloj < Reloj_Final && (repeticion <= Repeticiones1[umbral_act]));			
	// BUG: should be "Repeticiones" and not "Repeticiones1" here ^^^

	AcumulaTiemposUmbral(umbral_act,Tiempos_umbral);
}

/*****************************************************************************/
void Splitting(int umbral_inicial
			  , Tipo_Servidor Servidores[]
			  , Tipo_Tiempo_Umbral* Tiempos_umbral
			  , int ModoCortado
			  )
{
	int servidor; /* Servidor en el que ocurre el siguiente evento */
	int serv_salida = -1;
	int serv_entrada = -1;
	int i;
	// PARA GUARDAR el estado con el que entra al umbral Y LUEGO PODER RECUPERARLO
	Tipo_Servidor Servidor_Guardado[Num_Servidores];
	// PARA GUARDAR EL RELOJ CON EL QUE SE ENTRA AL UMBRAL Y LUEGO PODER RECUPERARLO
	double reloj_Guardado = 0;

	double importancia; /* guarda el calculo de la funcion de importancia */
	int repeticion = 1;   /* numero de repeticiones que se realizan en el umbral actual*/

 	/* guardo el estado actual */
	for (i = 0; i < Num_Servidores; i++)
	{		
		 Servidor_Guardado[i].Estado = Servidores[i].Estado;
		 Servidor_Guardado[i].Num_Clientes = Servidores[i].Num_Clientes;
		 Servidor_Guardado[i].Cliente = Servidores[i].Cliente;
		 Servidor_Guardado[i].Suceso_Ant = Servidores[i].Suceso_Ant;		 
	}
	reloj_Guardado = reloj;
	double reloj_Anterior = 0;
	reloj_Anterior = reloj_Guardado;	
	int Viene_De_Fuera = FALSE;
	int BajoDelUmbral_Inicial = FALSE;	 			
	int umbral_act = umbral_inicial; 			
	Tipo_Tiempo_Umbral Tiempos_umbral_Despues_de_bajar;				 			
	int Ya_Hizo_Splitting = FALSE;	 				

	do  // while (repeticion <= Repeticiones1[umbral_inicial]);		
 	{
		Viene_De_Fuera = FALSE; 		
		BajoDelUmbral_Inicial = FALSE;	 		
		Ya_Hizo_Splitting = FALSE;
		umbral_act = umbral_inicial; 		

		for (i = 0; i <= Numero_Umbrales; i++)
		{
			 Tiempos_umbral_Despues_de_bajar.Tiempos[i] = 0.0;
		}
		Tiempos_umbral_Despues_de_bajar.reloj = 0.0;
		Tiempos_umbral_Despues_de_bajar.suceso_ant = 0.0; 		
		
 		do // while (reloj < Reloj_Final);				
 		{
	 	    // SE CALCULA LA PROXIMA LLEGADA Y SALIDA Y SE MIRA A VER CUAL SE VA A DAR ANTES 		
			if (Proxima_Salida(&serv_salida,Servidores) <= Proxima_Llegada(&serv_entrada,Servidores))
			{
			  // SI LA Proxima_Salida es menor o igual que la Proxima_Llegada HACE SALIDA			
			  servidor=serv_salida;
			  if (BajoDelUmbral_Inicial)
				{
					Gestiona_Salida(servidor
									,umbral_act
									,umbral_inicial
									,Servidores
									,&Tiempos_umbral_Despues_de_bajar
									,TRUE);
				}
			  else
				{							  
			  		Gestiona_Salida(servidor
					  				  ,umbral_act
					  				  ,umbral_inicial
									  ,Servidores
									  ,Tiempos_umbral
									  ,FALSE);
				}
			}
			else
			{
			  // SI LA Proxima_Salida es MAYOR que la Proxima_Llegada HACE LLEGADA				
			  servidor=serv_entrada;
			  Viene_De_Fuera = TRUE;
			  if (BajoDelUmbral_Inicial)
				{
			  		Gestiona_Llegada(servidor
					  ,Viene_De_Fuera
					  ,umbral_act
					  ,umbral_inicial
					  ,Servidores
					  ,&Tiempos_umbral_Despues_de_bajar
					  ,TRUE);
				}
			  else
				{							  
			  		Gestiona_Llegada(servidor
					  ,Viene_De_Fuera
					  ,umbral_act
					  ,umbral_inicial
					  ,Servidores
					  ,Tiempos_umbral
					  ,FALSE);
				}
			  Clientes_Servidos++;
			}
	
			/* calculo la funcion de importancia */
			// EN FUNCION DEL NUMERO DE CLIENTES EN CADA SERVIDOR
			importancia = Calcula_Importancia(Servidores);
		
        	
			/* en funcion de la importancia cambio o no de umbral*/
			if (
				(umbral_inicial == umbral_act)
				&& (umbral_inicial < Numero_Umbrales) 
				&& (importancia > Umbrales[umbral_inicial + 1])		   
			    )
			     
			{
				// LA IMPORTANCIA CALCULADA ES MAYOR QUE LA DEL UMBRAL SIGUIENTE
				/* Sube de umbral SIGUIENTE AL INICIAL*/
				if (BajoDelUmbral_Inicial)								
				{
					AcumulaTiemposUmbral_DespuesDeBajar(umbral_act,Tiempos_umbral,&Tiempos_umbral_Despues_de_bajar);																
					if (Ya_Hizo_Splitting == FALSE)						
						{												
						reloj_Anterior = reloj;  
						}	
				}
				else
				{
					AcumulaTiemposUmbral(umbral_inicial,Tiempos_umbral);			
				}				
				/* Llamada recursiva para el siguiente nivel */
				// YA QUE LLEGO A LA IMPORTANCIA DEL SIGUIENTE UMBRAL
				// Y NO HE LLEGADO AL NUMERO TOTAL DE UMBRALES					
				Splitting(umbral_inicial + 1, Servidores, Tiempos_umbral, ModoCortado);	
				Ya_Hizo_Splitting = TRUE;	  
			}
			else 
			
			{			
				if (BajoDelUmbral_Inicial)
				{
					if (importancia < Umbrales[umbral_act])
					{
						AcumulaTiemposUmbral_DespuesDeBajar(umbral_act,Tiempos_umbral, &Tiempos_umbral_Despues_de_bajar);					
						if (Ya_Hizo_Splitting == FALSE )						
							{							
								reloj_Anterior = reloj;							
							}	

						if (umbral_act > 0)						
						{
							do
							{
								if (umbral_act > 0)  
								{
									umbral_act--;							
								}								
							} while (importancia < Umbrales[umbral_act]);
						}
					}
					else
					{
						if (importancia > Umbrales[umbral_act + 1])
						{
							AcumulaTiemposUmbral_DespuesDeBajar(umbral_act,Tiempos_umbral, &Tiempos_umbral_Despues_de_bajar);						
							if (Ya_Hizo_Splitting == FALSE )													
							{
								reloj_Anterior = reloj;							
							}							
							
							if (umbral_act < Numero_Umbrales)
							{
								umbral_act++;						
							}
						}
					}					
				}				
				else
				{
					if (importancia < Umbrales[umbral_act])
					{
         				if (umbral_inicial == umbral_act)                                    
         				{
    						BajoDelUmbral_Inicial = TRUE;
    						if ((Tiempos_umbral_Despues_de_bajar.umbral_calculo[umbral_act]) == 0)    						
    						{
              						Tiempos_umbral_Despues_de_bajar.umbral_calculo[umbral_act] = umbral_inicial;              						
                            }
                        } 
						AcumulaTiemposUmbral(umbral_inicial,Tiempos_umbral);			                        												
						do
						{
							if (umbral_act > 0)  
							{
								umbral_act--;							
							}								
						} while (importancia < Umbrales[umbral_act]);						
					}	
					
				}
			}
	} while ((reloj < Reloj_Final) 
				&& (!Ya_Hizo_Splitting) 
				&& (
					(ModoCortado == FALSE)				
					||
					(ModoCortado == TRUE) && ((umbral_inicial - umbral_act) < MaxUmbralesBajadosSplitting)
				   )
					
			);
	
		/* Calcula tiempo en el umbral ultimo */
		if (BajoDelUmbral_Inicial)				
		{
			AcumulaTiemposUmbral_DespuesDeBajar(umbral_act,Tiempos_umbral, &Tiempos_umbral_Despues_de_bajar);														
			if (Ya_Hizo_Splitting == FALSE )																
			{
				reloj_Anterior = reloj;						
			}			
		}
		
	 	if (repeticion < Repeticiones1[umbral_inicial])
 		{
 		// SI NO HE LLEGADO AL NUMERO DE REPETICIONES DEL UMBRAL
		// recuperamos reloj y estado servidores AL ENTRAR AL UMBRAL
		// Y HACEMOS RE-SCHEDULING ANTES DE REPETIR
			reloj = reloj_Guardado;
			reloj_Anterior = reloj_Guardado;	
			for (i = 0; i < Num_Servidores; i++)
			 {
				 Servidores[i].Estado = Servidor_Guardado[i].Estado;
				 Servidores[i].Num_Clientes = Servidor_Guardado[i].Num_Clientes;
				 Servidores[i].Cliente = Servidor_Guardado[i].Cliente;
			 	 Servidores[i].Suceso_Ant = Servidor_Guardado[i].Suceso_Ant;						
		 	}
		 	Lista_Sucesos_Nueva(Servidores);		 	
 			umbral_act = umbral_inicial;			 	
			BajoDelUmbral_Inicial = FALSE;					 			
			for (i = 0; i <= Numero_Umbrales; i++)
			{
				 Tiempos_umbral_Despues_de_bajar.Tiempos[i]=0.0;
			}
			Tiempos_umbral_Despues_de_bajar.reloj = 0.0;
			Tiempos_umbral_Despues_de_bajar.suceso_ant = 0.0; 		
			
 		}	
		repeticion++;	
	} while (repeticion <= Repeticiones1[umbral_inicial]);			
	
	double indice_umbral_despues;		
	for (i = 0; i <= Numero_Umbrales; i++)
 	{	
		indice_umbral_despues = Calcula_Indice_Umbral_Despues(umbral_inicial,i); 	
	}	
}

void Simulacion_Restart(int Posicion)
{
	int i;
	double prob,intervalo_confianza;
	double s = 0;
	double RM = 1;
	
	// ARRAY CON LOS SERVIDORES A SIMULAR
	Tipo_Servidor Servidores[Num_Servidores]; // 7 = 0..6
	
	Tipo_Tiempo_Umbral Tiempos_umbral;		

	/*Inicializo variable de probabilidades calculadas en cada muestra*/
	for (i = 0; i < Max_Muestras; i++)
		Probabilidades[i] = 0;

	// INICIALIZACION NUMERO DE MUESTRAS TRATADAS
	Van_Num_Muestras[Posicion] = 0;
	NumVecesSinClientes[Posicion] = 0;	
	NumVecesImportanciaCero = 0;
	
	Error_Relativo[Posicion] = 1; /*  Para que cumpla la condicion las primeras veces*/

	do
 	{
		Transitorio(Servidores,&Tiempos_umbral);		
		
		// COMIENZA SIMULACION RESTART DESDE EL UMBRAL 0
		Restart(0,Servidores,&Tiempos_umbral);		

		prob = Calcula_Probabilidad(Servidores,&Tiempos_umbral);		
		Probabilidades[Van_Num_Muestras[Posicion]] = prob;
	
		//Probabilidades Ti	
		for(i = 1; i <= Numero_Umbrales; i++)
		{	
    	   Probabilidades_Rep[Van_Num_Muestras[Posicion]][i-1] = Calcula_Probabilidad_Ti(&Tiempos_umbral,i);    	   
	    }
	    
    	//probabilidad TM+1
    	Probabilidades_Rep[Van_Num_Muestras[Posicion]][Numero_Umbrales] = prob;
	
		for (i = 0; i < (Numero_Umbrales + 1); i++)
		{
    	    Media_Pi[i][Posicion] = Calcula_Media_Umbral(Van_Num_Muestras[Posicion],i + 1);
     	}
	
		Van_Num_Muestras[Posicion]++;
		if (Van_Num_Muestras[Posicion] > Calcula_Error)
		{
	  		Media[Posicion] = Calcula_Media(Van_Num_Muestras[Posicion]);
	  		s = sqrt(Calcula_s2(Van_Num_Muestras[Posicion], Media[Posicion]));
	  		Error_Relativo[Posicion] = Calcula_Error_Relativo(s, Media[Posicion], Van_Num_Muestras[Posicion]);
		}

		Imprime_Pantalla(Van_Num_Muestras[Posicion], Error_Relativo[Posicion], Media[Posicion]);
    
 	} while ((Van_Num_Muestras[Posicion] < Max_Muestras) && (Error_Relativo[Posicion] > Max_Error));

	//Calculo ri 
	for (i = 0; i < Numero_Umbrales; i++)
 	{
    	printf("P%i:  %e   \n",i+1 ,Media_Pi[i][Posicion] * 2);
     	ri[i][Posicion] = 1/sqrt((Media_Pi[i+1][Posicion]/Media_Pi[0][Posicion]) *  Media_Pi[i][Posicion] * 2);
	}
    ri[Numero_Umbrales-1][Posicion] = 1/sqrt((Media_Pi[Numero_Umbrales][Posicion]/Media_Pi[0][Posicion]) *  Media_Pi[Numero_Umbrales][Posicion] * 2);
    
	//Calculo Ri 
	Ri[0][Posicion] = (int)round(ri[0][Posicion]);
  	for (i = 1; i < Numero_Umbrales; i++)
 	{
    	RM = RM*(double)Ri[i-1][Posicion];
     	Ri[i][Posicion] = (int)round((ri[i][Posicion]/RM));
	} 
	
	NumVecesSinClientes[Posicion] = NumVecesImportanciaCero;	
 
	for (i = 0; i < Numero_Umbrales; i++)
 	{
    	printf("Repeticiones para el umbral %i:  %i \n",(i+1),Ri[i][Posicion]);
	} 

 	intervalo_confianza = Calcula_Intervalo_Confianza(s,Van_Num_Muestras[Posicion]);
 	extremo_inf[Posicion] = Media[Posicion] - intervalo_confianza;
 	extremo_sup[Posicion] = Media[Posicion] + intervalo_confianza; 
 	printf("Intervalo de confianza: ( %e , %e )  \n",extremo_inf[Posicion],extremo_sup[Posicion]);
}

void Simulacion_Splitting(int Posicion
						, int ModoCortado
						)
{
	int i;
	double prob,intervalo_confianza;
	double s = 0;
	double RM = 1;
	
	// ARRAY CON LOS SERVIDORES A SIMULAR
	Tipo_Servidor Servidores[Num_Servidores]; // 7 = 0..6
	
	Tipo_Tiempo_Umbral Tiempos_umbral;		

	for (i = 0; i <= Numero_Umbrales; i++)
 	{	
		Tiempos_umbral.umbral_calculo[i] = 0;
	}

	/*Inicializo variable de probabilidades calculadas en cada muestra*/
	for (i = 0; i < Max_Muestras; i++)
		Probabilidades[i] = 0;

	// INICIALIZACION NUMERO DE MUESTRAS TRATADAS
	Van_Num_Muestras[Posicion] = 0;
	NumVecesSinClientes[Posicion] = 0;		
	NumVecesImportanciaCero = 0;	
	
	Error_Relativo[Posicion] = 1; /*  Para que cumpla la condicion las primeras veces*/

	do
 	{
		Transitorio(Servidores,&Tiempos_umbral);		
		
		// COMIENZA SIMULACION SPLITTIING DESDE EL UMBRAL 0
		Splitting(0,Servidores,&Tiempos_umbral,ModoCortado);			

		prob = Calcula_Probabilidad_Splitting(Servidores,&Tiempos_umbral);				
		Probabilidades[Van_Num_Muestras[Posicion]] = prob;
	
		//Probabilidades Ti	
		for(i = 1; i <= Numero_Umbrales; i++)
		{	
    	   Probabilidades_Rep[Van_Num_Muestras[Posicion]][i-1] = Calcula_Probabilidad_Ti_Splitting(&Tiempos_umbral,i);    	       	   
	    }
	    
    	//probabilidad TM+1
    	Probabilidades_Rep[Van_Num_Muestras[Posicion]][Numero_Umbrales] = prob;
	
		for (i = 0; i < (Numero_Umbrales + 1); i++)
		{
    	    Media_Pi[i][Posicion] = Calcula_Media_Umbral(Van_Num_Muestras[Posicion],i + 1);
     	}
	
		Van_Num_Muestras[Posicion]++;
		if (Van_Num_Muestras[Posicion] > Calcula_Error)
		{
	  		Media[Posicion] = Calcula_Media(Van_Num_Muestras[Posicion]);
	  		s = sqrt(Calcula_s2(Van_Num_Muestras[Posicion], Media[Posicion]));
	  		Error_Relativo[Posicion] = Calcula_Error_Relativo(s, Media[Posicion], Van_Num_Muestras[Posicion]);
		}
	
		Imprime_Pantalla(Van_Num_Muestras[Posicion], Error_Relativo[Posicion], Media[Posicion]);
    
 	} while ((Van_Num_Muestras[Posicion] < Max_Muestras) && (Error_Relativo[Posicion] > Max_Error));

	//Calculo ri 
	for (i = 0; i < Numero_Umbrales; i++)
 	{
    	printf("P%i:  %e   \n",i+1 ,Media_Pi[i][Posicion] * 2);
     	ri[i][Posicion] = 1/sqrt((Media_Pi[i+1][Posicion]/Media_Pi[0][Posicion]) *  Media_Pi[i][Posicion] * 2);
	}
    ri[Numero_Umbrales-1][Posicion] = 1/sqrt((Media_Pi[Numero_Umbrales][Posicion]/Media_Pi[0][Posicion]) *  Media_Pi[Numero_Umbrales][Posicion] * 2);
    
	//Calculo Ri 
	Ri[0][Posicion] = (int)round(ri[0][Posicion]);
  	for (i = 1; i < Numero_Umbrales; i++)
 	{
    	RM = RM*(double)Ri[i-1][Posicion];
     	Ri[i][Posicion] = (int)round((ri[i][Posicion]/RM));
	} 
	
	NumVecesSinClientes[Posicion] = NumVecesImportanciaCero;		
 
	for (i = 0; i < Numero_Umbrales; i++)
 	{
    	printf("Repeticiones para el umbral %i:  %i \n",(i+1),Ri[i][Posicion]);
	} 

 	intervalo_confianza = Calcula_Intervalo_Confianza(s,Van_Num_Muestras[Posicion]);
 	extremo_inf[Posicion] = Media[Posicion] - intervalo_confianza;
 	extremo_sup[Posicion] = Media[Posicion] + intervalo_confianza; 
 	printf("Intervalo de confianza: ( %e , %e )  \n",extremo_inf[Posicion],extremo_sup[Posicion]);
}

/*****************************************************************************/
/*                           PROGRAMA PRINCIPAL                              */
/*****************************************************************************/
int main(void)
{	
	time_t Tiempo_Ini[Num_Simulaciones]
		 , Tiempo_Fin[Num_Simulaciones]
		 , Tiempo_Total[Num_Simulaciones];
		 
	int NumSegundos[Num_Simulaciones]
	  , NumMinutos[Num_Simulaciones]
	  , NumHoras[Num_Simulaciones];
	  
    int i,j, VanEjecuciones;
    
	struct tm *HoraActual;	
	time_t curtime;
  	curtime = time (NULL);
  	HoraActual = localtime (&curtime);  	
  	
    string nombre = "runned_";
    
	ostringstream temp;      
	string variable1;		
    string variable2;    	
    
    int anio = HoraActual->tm_year + 1900;
    int mes = HoraActual->tm_mon + 1;        
    int dia = HoraActual->tm_mday;            
    
	int hora = HoraActual->	tm_hour;
	int minuto = HoraActual->tm_min;
	int segundo = HoraActual->tm_sec;
    
	temp<<anio;
	
	if (mes<10) temp << "0";
	temp<<mes;
	
	if (dia<10) temp << "0";	
	temp<<dia;	

	temp<<"_";
	
	if (hora<10) temp << "0";		
	temp<<hora;
	
	if (minuto<10) temp << "0";		
	temp<<minuto;

	if (segundo<10) temp << "0";	
	temp<<segundo;	
	
	variable1 = temp.str();    

    string extension = ".txt";
    string fichero = nombre + variable1 + extension;
 	ofstream myfile;
 	myfile.open(fichero.c_str());
 	
 	myfile << "RESTART;";	 	
 	myfile << ";";	 	 	
 	myfile << ";";	 	 	 	
 	myfile << ";";	 	 	
 	myfile << ";";	 	 	 	
 	myfile << ";";	 	 	 	
 	
 	myfile << "SPLITTING;";	 	
 	myfile << ";";	 	 	
 	myfile << ";";	
 	myfile << ";";	 	 	
 	myfile << ";";	 	 	 	
 	myfile << ";";	 	 	 	
 	
 	myfile << "SPLITTING-CORTADO;";	 	
 	myfile << ";";	 	 	
 	myfile << ";";	
 	myfile << ";";	 	 	
 	myfile << ";";	 	 	 	
 	myfile << ";"; 	
 	
 	myfile << "\n";	 	 	 	 	
    
 	myfile << "Muestra;";	 	 	 	 	    
 	myfile << "Error;";	 	 	 	 	    
	myfile << "Media;";	 	 
    myfile << "Num Sin;";        		 	 	    
 	myfile << "Intervalo de confianza;";		
 	myfile << "Tiempo Ejecucion;";		 	
	
 	myfile << "Muestra;";	 	 	 	 	    
 	myfile << "Error;";	 	 	 	 	    
	myfile << "Media;";	 	 	 	
    myfile << "Num Sin;";        		 	 	    	
 	myfile << "Intervalo de confianza;";	
 	myfile << "Tiempo Ejecucion;";		 		 	
 	
 	myfile << "Muestra;";	 	 	 	 	    
 	myfile << "Error;";	 	 	 	 	    
	myfile << "Media;";	 	 	 	
    myfile << "Num Sin;";        		 	 	    	
 	myfile << "Intervalo de confianza;";	
 	myfile << "Tiempo Ejecucion;";		 		 	
	 	    
 	myfile << "\n";	 	 	 	 	  	
    
	
	for (VanEjecuciones = 1; VanEjecuciones <= Num_Ejecuciones; VanEjecuciones++)
	{

	for (j = 0; j < Num_Simulaciones; j++)
	{ 		
		for (i = 0; i <= Numero_Umbrales; i++)
		{
		Media_Pi[i][j] = 0;
		}
		
		for (i = 0; i < Numero_Umbrales - 1; i++)
		{		
		ri[i][j] = 0;		
		Ri[i][j] = 0;		
		}	
	
		extremo_inf[j] = 0;	
		extremo_sup[j] = 0;
		Error_Relativo[j] = 0; /* Calculo del error relativo */
		Media[j] = 0; /* Media de la muestras (parmetro a estimar) */
	}	
	
  	//      123456789.123456789.123456789.123456789.123456789.123456789.123456789.123456789.
  	//      .........1.........2.........3.........4.........5.........6.........7.........8
    printf("------------- SIMULACION JACKSON x NODOS RESTART vs SPLITTING y SPLITTING-CORTADO -----------------\n");		
    printf("Ejecucion: %s", asctime (HoraActual));
    printf("- CONDICIONES DE SIMULACION -\n");		    
    printf("Num_Servidores %i \n", Num_Servidores);
    printf("Tam_Muestra %i \n", Tam_Muestra);
    printf("Tam_Transitorio %i \n", Tam_Transitorio);    
    printf("Servidor_Simulado %i \n", Servidor_Simulado);     
    printf("Clientes_En_Servidor %i \n", Clientes_En_Servidor);    
    printf("Numero_Umbrales %i \n", Numero_Umbrales);    
    printf("Max_Muestras %i \n", Max_Muestras);    
    printf("Max_Error %e \n", Max_Error);    
    printf("Numero de muestras a partir de las que empieza a calcular el error relativo %i \n",Calcula_Error);        
    printf("Tiempo Limite de Simulacin %i \n",Reloj_Final);            
    printf("Numero de Umbrales Maximos a Bajar en Splitting-Cortado %i \n",MaxUmbralesBajadosSplitting);                

    printf("------------- RESTART  -----------------\n");		
	// INICIALIZA CON EL NUMERO DE MILISEGUNDOS QUE LLEVA ENCENDIDO EL EQUIPO EL GENERADOR DE NUMEROS ALEATORIOS
	// QUE SERA DIFERENTE PARA CADA EJECUCION DEL PROGRAMA
	srand(time(NULL));	  
	
	VanPosicion = 0;	  
	// INICIALIZAMOS CUANDO COMIENZA LA SIMULACION
	Tiempo_Ini[VanPosicion] = time(NULL);			
	Simulacion_Restart(VanPosicion);			
	// ANOTAMOS CUANDO TERMINA LA SIMULACION
	Tiempo_Fin[VanPosicion] = time(NULL);
	
	// TIEMPO TOTAL DE LA SIMULACION
	Tiempo_Total[VanPosicion] = Tiempo_Fin[VanPosicion] - Tiempo_Ini[VanPosicion];	
	
	NumSegundos[VanPosicion] = Tiempo_Total[VanPosicion]%60;
	Tiempo_Total[VanPosicion] /=60; // PIERDE LOS DECIMALES
	
	NumMinutos[VanPosicion] = Tiempo_Total[VanPosicion]%60;
	Tiempo_Total[VanPosicion] /=60; // PIERDE LOS DECIMALES
	
	NumHoras[VanPosicion] = Tiempo_Total[VanPosicion]%60;
	Tiempo_Total[VanPosicion] /=60; // PIERDE LOS DECIMALES
    printf("Tiempo total RESTART %i horas %i minutos %i segundos \n", NumHoras[VanPosicion], NumMinutos[VanPosicion], NumSegundos[VanPosicion]);	
    
 /*   printf("------------- SPLITTING  -----------------\n");		    
	// INICIALIZA CON EL NUMERO DE MILISEGUNDOS QUE LLEVA ENCENDIDO EL EQUIPO EL GENERADOR DE NUMEROS ALEATORIOS
	// QUE SERA DIFERENTE PARA CADA EJECUCION DEL PROGRAMA
	srand(time(NULL));	      
    
	VanPosicion = 1;	  
	// INICIALIZAMOS CUANDO COMIENZA LA SIMULACION
	Tiempo_Ini[VanPosicion] = time(NULL);			
	Simulacion_Splitting(VanPosicion,FALSE);				
	// ANOTAMOS CUANDO TERMINA LA SIMULACION
	Tiempo_Fin[VanPosicion] = time(NULL);
	
	// TIEMPO TOTAL DE LA SIMULACION
	Tiempo_Total[VanPosicion] = Tiempo_Fin[VanPosicion] - Tiempo_Ini[VanPosicion];	
	
	NumSegundos[VanPosicion] = Tiempo_Total[VanPosicion]%60;
	Tiempo_Total[VanPosicion] /=60; // PIERDE LOS DECIMALES
	
	NumMinutos[VanPosicion] = Tiempo_Total[VanPosicion]%60;
	Tiempo_Total[VanPosicion] /=60; // PIERDE LOS DECIMALES
	
	NumHoras[VanPosicion] = Tiempo_Total[VanPosicion]%60;
	Tiempo_Total[VanPosicion] /=60; // PIERDE LOS DECIMALES
    printf("Tiempo total SPLITTING %i horas %i minutos %i segundos \n", NumHoras[VanPosicion], NumMinutos[VanPosicion], NumSegundos[VanPosicion]);	
    
    
    printf("------------- SPLITTING-CORTADO  -----------------\n");		    
	// INICIALIZA CON EL NUMERO DE MILISEGUNDOS QUE LLEVA ENCENDIDO EL EQUIPO EL GENERADOR DE NUMEROS ALEATORIOS
	// QUE SERA DIFERENTE PARA CADA EJECUCION DEL PROGRAMA
	srand(time(NULL));	      
    
	VanPosicion = 2;	  
	// INICIALIZAMOS CUANDO COMIENZA LA SIMULACION
	Tiempo_Ini[VanPosicion] = time(NULL);			
	Simulacion_Splitting(VanPosicion,TRUE);				
	// ANOTAMOS CUANDO TERMINA LA SIMULACION
	Tiempo_Fin[VanPosicion] = time(NULL);
	
	// TIEMPO TOTAL DE LA SIMULACION
	Tiempo_Total[VanPosicion] = Tiempo_Fin[VanPosicion] - Tiempo_Ini[VanPosicion];	
	
	NumSegundos[VanPosicion] = Tiempo_Total[VanPosicion]%60;
	Tiempo_Total[VanPosicion] /=60; // PIERDE LOS DECIMALES
	
	NumMinutos[VanPosicion] = Tiempo_Total[VanPosicion]%60;
	Tiempo_Total[VanPosicion] /=60; // PIERDE LOS DECIMALES
	
	NumHoras[VanPosicion] = Tiempo_Total[VanPosicion]%60;
	Tiempo_Total[VanPosicion] /=60; // PIERDE LOS DECIMALES
    printf("Tiempo total SPLITTING-CORTADO %i horas %i minutos %i segundos \n", NumHoras[VanPosicion], NumMinutos[VanPosicion], NumSegundos[VanPosicion]);	
*/
  	//      123456789.123456789.123456789.123456789.123456789.123456789.123456789.123456789.
  	//      .........1.........2.........3.........4.........5.........6.........7.........8    
    printf("Umbral;");    
    printf("RESTART;");
    printf(";");    
    
    printf("SPLITTING;");
    printf(";");        
    
    printf("SPLITTING-CORTADO;");
    printf(";");    
    
    printf("\n");
    
    printf(";");
    printf("Probabilidad;");
    printf("Repeticiones;");
    
    printf("Probabilidad;");
    printf("Repeticiones;");

    printf("Probabilidad;");
    printf("Repeticiones;");
    
    printf("\n");
    
	for (i = 0; i < Numero_Umbrales; i++)
 	{
    	printf("%i;",i + 1); 		 		 		
 		for (j = 0; j < Num_Simulaciones; j++)
 		{ 		
    		printf("%e;", Media_Pi[i][j] * 2);
    		printf("%i;", Ri[i][j]);    		
    	}	
    	printf("\n");    	
	}
    
 	printf("Intervalo de confianza;");		
    printf("RESTART;");
    printf(";");        
    printf("SPLITTING;");
    printf(";");        
    printf("SPLITTING-CORTADO;");
    printf(";");        
    
    printf("\n");    
    
    printf(";");    
	for (j = 0; j < Num_Simulaciones; j++)
	{	
 		printf("( %e , %e );", extremo_inf[j],extremo_sup[j]);	
    	printf(";");     		
 	}
    printf("\n");    	    	 	
    
 	printf("Tiempo Ejeucion;");		
    printf("RESTART;");
    printf(";");        
    printf("SPLITTING;");
    printf(";");        
    printf("SPLITTING-CORTADO;");
    printf(";");        
    
    printf("\n");    
    
    printf(";");    
	for (j = 0; j < Num_Simulaciones; j++)
	{	
    	printf("%i horas %i minutos %i segundos ;", NumHoras[j], NumMinutos[j], NumSegundos[j]);		 		
    	printf(";");     		
 	}
    printf("\n");    	    	 	
    
 	printf(";");		
 	
    printf("RESTART;");
    printf(";");        
    printf(";");            
    printf(";");                
    
    printf("SPLITTING;");
    printf(";");        
    printf(";");            
    printf(";");                
    
    printf("SPLITTING-CORTADO;");
    printf(";");        
    printf(";");            
    printf(";");                
    
    
    printf("\n");    
    
 	printf(";");		    
    
    printf("Muestra;");
    printf("Error;");
    printf("Media;");
    printf("Num Sin;");    
    
    printf("Muestra;");
    printf("Error;");
    printf("Media;");
    printf("Num Sin;");        
    
    printf("Muestra;");
    printf("Error;");
    printf("Media;");
    printf("Num Sin;");        
    
    
    printf("\n");    
    
    printf(";");    
	for (j = 0; j < Num_Simulaciones; j++)
	{	
    	printf("%i;",Van_Num_Muestras[j]);
    	printf("%.4f;",Error_Relativo[j]);
    	printf("%e;",Media[j]);
    	printf("%i;",NumVecesSinClientes[j]);    	
 	}
    printf("\n");    	    	 	
   
    
	for (j = 0; j < Num_Simulaciones; j++)
	{	
 		myfile << Van_Num_Muestras[j] << ";";		
 		myfile << Error_Relativo[j] << ";";		 		
 		myfile << Media[j] << ";";		 		 	
    	myfile << NumVecesSinClientes[j]<< ";";		 		 	
 		myfile << "(" << extremo_inf[j] << "," << extremo_sup[j] << ");";	 
 		myfile << NumHoras[j] << " horas " << NumMinutos[j] << " minutos " << NumSegundos[j] << " segundos;";
 	}
 	myfile << "\n";	 	 	 	 	        
 	
 	} // DE VanEjecuciones
	     	
 	myfile << "\n";	 	 	 	 	        
 	myfile << "\n";	 	 	 	 	        	 	     	
 	myfile << "\n";	 	 	 	 	         	
 	
    myfile << "------------- SIMULACION JACKSON x NODOS RESTART vs SPLITTING Y SPLITTING-CORTADO  -----------------"  << ";";		
 	myfile << "\n";	 	 	 	 	         	  
 	
    myfile << "Ejecucion:" << asctime (HoraActual)  << ";";			   
 	myfile << "\n";	 	 	 	 	         	      

    myfile << "- CONDICIONES DE SIMULACION -"<< ";";
 	myfile << "\n";	 	 	 	 	         	      				   
			    
    myfile << "Num_Servidores " <<  Num_Servidores << ";";
 	myfile << "\n";	 	 	 	 	         	      				   
	     
    myfile << "Tam_Muestra " <<  Tam_Muestra << ";";
 	myfile << "\n";	 	 	 	 	         	      				   
	     
    myfile << "Tam_Transitorio " << Tam_Transitorio << ";";
 	myfile << "\n";	 	 	 	 	         	      				       
	    
    myfile << "Servidor_Simulado " << Servidor_Simulado << ";";     
 	myfile << "\n";	 	 	 	 	         	      				       
	     
    myfile << "Clientes_En_Servidor " << Clientes_En_Servidor << ";";     
 	myfile << "\n";	 	 	 	 	         	      				       
	     
    myfile << "Numero_Umbrales " << Numero_Umbrales << ";";     
 	myfile << "\n";	 	 	 	 	         	      				       
	     
    myfile << "Max_Muestras " << Max_Muestras << ";";     
 	myfile << "\n";	 	 	 	 	         	      				       
	     
    myfile << "Max_Error " << Max_Error << ";";     
 	myfile << "\n";	 	 	 	 	         	      				       
	     
    myfile << "Numero de muestras a partir de las que empieza a calcular el error relativo " << Calcula_Error << ";";     
 	myfile << "\n";	 	 	 	 	         	      				           
    
    myfile << "Tiempo Limite de Simulacin " << Reloj_Final << ";";     
 	myfile << "\n";	 	 	 	 	         	      				               
 	
    myfile << "Numero de Umbrales Maximos a Bajar en Splitting-Cortado" << MaxUmbralesBajadosSplitting << ";";     
 	myfile << "\n";	 	 	 	 	         	      				                   
	     	
 	myfile.close(); 	
 	
	printf("*** Programa Finalizado ***");  
	getch();
}



