#!/usr/bin/env bash

export RESULTS_DIR="./results/modes";
mkdir -p ${RESULTS_DIR};
echo "Saving results in $RESULTS_DIR";

if [ -z ${TEST_RUN+x} ] || [ $TEST_RUN -ne 1 ]; then
	# Full reproduction
	for i in 1 2 3 4 5
	do
		for j in 0 1 2 3 4
		do
			modest modes models/2tandem_queue_Jackson.modest --max-run-length 0 --batch-size 1 --relative-width --width 0.005 --props SteadyState --ifun-prop Importance 0 30 --levels Manual "2:3,3:2,4:2,5:3,6:3,7:3,8:2,9:3,10:3,11:3,12:3,13:3,14:2,15:3,16:3,17:3,18:3,19:3,20:3,21:3,22:3,23:3,24:3,25:3,26:3,27:3,28:3,29:5" --lra 40 30000 -O ${RESULTS_DIR}/2tandem_queue_Jackson.P${j}.${i}.txt --rare Restart ${j}
		done
		for j in 0 1 2
		do
			modest modes models/3tandem_queue_Jackson.modest --max-run-length 0 --batch-size 1 --relative-width --width 0.1 --props SteadyState --ifun-prop Importance 0 34 --levels Manual "2:2,3:2,4:2,5:2,6:2,7:3,8:2,9:3,10:3,11:3,12:2,13:3,14:3,15:3,16:2,17:3,18:3,19:3,20:2,21:3,22:3,23:3,24:3,25:3,26:2,27:3,28:3,29:3,30:3,31:2,32:3,33:3,34:3" --lra 20 3000 -O ${RESULTS_DIR}/3tandem_queue_Jackson.P${j}.${i}.txt --rare Restart ${j}
			modest modes models/3tandem_queue_nonJackson.modest --max-run-length 0 --batch-size 1 --relative-width --width 0.1 --props SteadyState --ifun-prop Importance 0 160 --levels Manual "15:2,25:3,30:2,35:3,40:2,45:3,50:3,55:3,60:3,65:3,70:3,75:3,80:3,85:3,90:3,95:3,100:3,105:3,110:3,115:3,120:3,125:3,130:3,135:3,140:4,145:3,150:3,155:4,160:5" --lra 20 3000 -O ${RESULTS_DIR}/3tandem_queue_nonJackson.P${j}.${i}.txt --rare Restart ${j}
			modest modes models/7nodes_network_Jackson.modest --max-run-length 0 --batch-size 1 --relative-width --width 0.1 --props SteadyState --ifun-prop Importance 0 32 --levels Manual "2:2,3:2,4:3,5:3,6:3,7:3,8:3,9:3,10:3,11:3,12:3,13:3,14:3,15:3,16:3,17:3,18:3,19:4,20:3,21:3,22:3,23:3,24:3,25:3,26:3,27:3,28:3,29:3,30:3,31:3,32:2" --lra 6 500 -O ${RESULTS_DIR}/7nodes_network_Jackson.P${j}.${i}.txt --rare Restart ${j}
			modest modes models/7nodes_network_nonJackson.modest --max-run-length 0 --batch-size 1 --relative-width --width 0.1 --props SteadyState --ifun-prop Importance 0 29 --levels Manual "2:2,3:2,4:2,5:3,6:3,7:3,8:3,9:3,10:3,11:3,12:2,13:3,14:3,15:3,16:3,17:2,18:3,19:4,20:3,21:3,22:2,23:3,24:3,25:3,26:3,27:2,28:3,29:4" --lra 6 500 -O ${RESULTS_DIR}/7nodes_network_nonJackson.P${j}.${i}.txt --rare Restart ${j}
		done
	done
else
	# Fast test run: no repetitions, fewer prolongation depth values, larger statistical error for 2tandem_queue_Jackson model
	for j in 0 2
	do
		modest modes models/2tandem_queue_Jackson.modest --max-run-length 0 --batch-size 1 --relative-width --width 0.1 --props SteadyState --ifun-prop Importance 0 30 --levels Manual "2:3,3:2,4:2,5:3,6:3,7:3,8:2,9:3,10:3,11:3,12:3,13:3,14:2,15:3,16:3,17:3,18:3,19:3,20:3,21:3,22:3,23:3,24:3,25:3,26:3,27:3,28:3,29:5" --lra 40 30000 -O ${RESULTS_DIR}/2tandem_queue_Jackson.P${j}.1.txt --rare Restart ${j}
	done
	for j in 0 2
	do
		modest modes models/3tandem_queue_Jackson.modest --max-run-length 0 --batch-size 1 --relative-width --width 0.1 --props SteadyState --ifun-prop Importance 0 34 --levels Manual "2:2,3:2,4:2,5:2,6:2,7:3,8:2,9:3,10:3,11:3,12:2,13:3,14:3,15:3,16:2,17:3,18:3,19:3,20:2,21:3,22:3,23:3,24:3,25:3,26:2,27:3,28:3,29:3,30:3,31:2,32:3,33:3,34:3" --lra 20 3000 -O ${RESULTS_DIR}/3tandem_queue_Jackson.P${j}.1.txt --rare Restart ${j}
		modest modes models/3tandem_queue_nonJackson.modest --max-run-length 0 --batch-size 1 --relative-width --width 0.1 --props SteadyState --ifun-prop Importance 0 160 --levels Manual "15:2,25:3,30:2,35:3,40:2,45:3,50:3,55:3,60:3,65:3,70:3,75:3,80:3,85:3,90:3,95:3,100:3,105:3,110:3,115:3,120:3,125:3,130:3,135:3,140:4,145:3,150:3,155:4,160:5" --lra 20 3000 -O ${RESULTS_DIR}/3tandem_queue_nonJackson.P${j}.1.txt --rare Restart ${j}
		modest modes models/7nodes_network_Jackson.modest --max-run-length 0 --batch-size 1 --relative-width --width 0.1 --props SteadyState --ifun-prop Importance 0 32 --levels Manual "2:2,3:2,4:3,5:3,6:3,7:3,8:3,9:3,10:3,11:3,12:3,13:3,14:3,15:3,16:3,17:3,18:3,19:4,20:3,21:3,22:3,23:3,24:3,25:3,26:3,27:3,28:3,29:3,30:3,31:3,32:2" --lra 6 500 -O ${RESULTS_DIR}/7nodes_network_Jackson.P${j}.1.txt --rare Restart ${j}
		modest modes models/7nodes_network_nonJackson.modest --max-run-length 0 --batch-size 1 --relative-width --width 0.1 --props SteadyState --ifun-prop Importance 0 29 --levels Manual "2:2,3:2,4:2,5:3,6:3,7:3,8:3,9:3,10:3,11:3,12:2,13:3,14:3,15:3,16:3,17:2,18:3,19:4,20:3,21:3,22:2,23:3,24:3,25:3,26:3,27:2,28:3,29:4" --lra 6 500 -O ${RESULTS_DIR}/7nodes_network_nonJackson.P${j}.1.txt --rare Restart ${j}
	done
fi

echo "Finished experiments: logs stored in $RESULTS_DIR/*";
echo "Compiling summary tables";

HARD_MODELS=(
	2tandem_queue_Jackson
)
SOFT_MODELS=(
	3tandem_queue_Jackson
	3tandem_queue_nonJackson
	7nodes_network_Jackson
	7nodes_network_nonJackson
)
if [ -z ${TEST_RUN+x} ] || [ $TEST_RUN -ne 1 ]; then
	# Full experimental reproduction
	NUM_REPETITIONS=5;
	HARD_P=(0 1 2 3 4);
	SOFT_P=(0 1 2);
	HARD_CI="95% 0.005";
	SOFT_CI="95% 0.1";
else
	# Fast test run
	NUM_REPETITIONS=1;
	HARD_P=(0 2);
	SOFT_P=(0 2);
	HARD_CI="95% 0.1";
	SOFT_CI="95% 0.1";
fi
CHART="$RESULTS_DIR/summary";
cat "summary_header.txt" > "$CHART";
get_values() {
	local ESTIMAT="`awk '/Estimated value/{ printf("%.2e"), $3 }' $1`";
	local SAMPLES="`awk '/Runs used/      { print "("$3")"     }' $1`";
	local RUNTIME="`awk '/Simulation time/{ printf("%.1f"), $3 }' $1`";
	echo -e "restart$2\t $ESTIMAT\t $SAMPLES\t\t $RUNTIME";
}
for M in "${HARD_MODELS[@]}"; do
	echo "$M  |  CI: $HARD_CI" >> $CHART;
	for P in "${HARD_P[@]}"; do
		echo >> "$CHART";
		for (( REP=1 ; REP<=NUM_REPETITIONS ; REP++ )); do
			LOG="$RESULTS_DIR/$M.P$P.$REP.txt";
			get_values "$LOG" $P >> "$CHART";
		done # this repetition
	done # this RESTART prolongation
	echo >> "$CHART";
done # this model
for M in "${SOFT_MODELS[@]}"; do
	echo "$M  |  CI: $SOFT_CI" >> $CHART;
	for P in "${SOFT_P[@]}"; do
		echo >> "$CHART";
		for (( REP=1 ; REP<=NUM_REPETITIONS ; REP++ )); do
			LOG="$RESULTS_DIR/$M.P$P.$REP.txt";
			get_values "$LOG" $P >> "$CHART";
		done # this repetition
	done # this RESTART prolongation
	echo >> "$CHART";
done # this model
mkdir -p "$RESULTS_DIR/logs";
mv "$RESULTS_DIR"/*.txt "$RESULTS_DIR/logs/";
mv $CHART{,.txt};

echo "Summary tables saved in $PWD/$CHART.txt";
echo "Finished experimental reproduction on `date "+%b %d @ %T"`";

exit 0;
