
# Assert that given file exists (exits 1 if not)
assert_file(){
	[ -f "$1" ] || (echo "File "$1" not found; aborting" && exit 1);
}

# Make sure that the 'fig' executable exists in the environment
assert_fig() {
	set -e;
	command -v fig &>/dev/null ||
		(echo "ERROR: fig not found in environment! Aborting" && exit 1);
	set +e;
}

# Poll until less than MAXJOBSN jobs called 'fig' are running
# Optionally pass part of the invocation command for thinner filtering
poll_till_free_FIG() {
	if [ -z $MAXJOBSN ] || [ $MAXJOBSN -le 0 ]; then
		local JOBSBOUND=`nproc --all`;
	else
		local JOBSBOUND=$MAXJOBSN;
	fi
	# do-while syntax sugar for bash
	while
		#local RUNNING=`ps -fC "fig" | grep "\<$1" | wc -l`
		local RUNNING=`ps -fC "fig" | grep "$1" | wc -l`
		[ $RUNNING -ge $JOBSBOUND ]
	do
		sleep 3s;
	done
	sleep 0.1;  # give previous job a little while to sink in
}

# Poll until less than MAXJOBSN jobs called 'RESTART' are running
poll_till_free_RESTART() {
	if [ -z $MAXJOBSN ] || [ $MAXJOBSN -le 0 ]; then
		local JOBSBOUND=`nproc --all`;
	else
		local JOBSBOUND=$MAXJOBSN;
	fi
	# do-while syntax sugar for bash
	while
		local RUNNING=`pgrep "RESTART" | wc -l`
		[ $RUNNING -ge $JOBSBOUND ]
	do
		sleep 3s;
	done
	sleep 0.1;  # give previous job a little while to sink in
}

# Create a fresh directory to store results into,
# prepending the value of variable RESULTS_DIR, if any
make_fresh_results_dir() {
	if [ -z ${RESULTS_DIR} ]; then
		local RESULTS_DIR=".";
	fi
	while
		local NOW=`date +%b%d_%T`
		[ -e "${RESULTS_DIR}/${NOW}" ]
	do
		sleep 1;
	done
	mkdir -p "${RESULTS_DIR}/${NOW}";
	echo "${RESULTS_DIR}/${NOW}";
}

return 0;
